r"""
Time series module
"""


from .main import *
from .main import __all__ as _main__all__

from .functions import *
from .functions import __all__ as _functions__all__

__all__ = (
    *_main__all__,
    *_functions__all__,
)

from .functions import functional_form_context

